@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2017, The pgAdmin Development Team
+# Copyright (C) 2013 - 2019, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -14,36 +14,40 @@ from functools import wraps
 
 import pgadmin.browser.server_groups.servers.databases as databases
 from flask import render_template, make_response, request, jsonify
-from flask_babel import gettext
+from flask_babelex import gettext
 from pgadmin.browser.collection import CollectionNodeModule
 from pgadmin.browser.server_groups.servers.utils import parse_priv_from_db, \
     parse_priv_to_db, validate_options, tokenize_options
 from pgadmin.browser.utils import PGChildNodeView
 from pgadmin.utils.ajax import make_json_response, internal_server_error, \
-    make_response as ajax_response
+    make_response as ajax_response, gone
 from pgadmin.utils.driver import get_driver
-from pgadmin.utils.ajax import gone
-
 from config import PG_DEFAULT_DRIVER
+from pgadmin.utils import IS_PY2
+# If we are in Python3
+if not IS_PY2:
+    unicode = str
 
 
 class ForeignServerModule(CollectionNodeModule):
     """
     class ForeignServerModule(CollectionNodeModule)
 
-        A module class for foreign server node derived from CollectionNodeModule.
+        A module class for foreign server node derived from
+        CollectionNodeModule.
 
     Methods:
     -------
     * __init__(*args, **kwargs)
-      - Method is used to initialize the Foreign server module and it's base module.
+      - Method is used to initialize the Foreign server module and
+        it's base module.
 
     * get_nodes(gid, sid, did)
       - Method is used to generate the browser collection node.
 
     * script_load(self)
-      - Load the module script for foreign server, when any of the database node is
-        initialized.
+      - Load the module script for foreign server, when any of
+        the database node is initialized.
     """
 
     NODE_TYPE = 'foreign_server'
@@ -51,7 +55,8 @@ class ForeignServerModule(CollectionNodeModule):
 
     def __init__(self, *args, **kwargs):
         """
-        Method is used to initialize the Foreign server module and it's base module.
+        Method is used to initialize the Foreign server module and
+        it's base module.
 
         Args:
             *args:
@@ -78,12 +83,21 @@ class ForeignServerModule(CollectionNodeModule):
     @property
     def script_load(self):
         """
-        Load the module script for foreign server, when any of the foreign data wrapper node is initialized.
+        Load the module script for foreign server, when any of the
+        foreign data wrapper node is initialized.
 
         Returns: node type of the server module.
         """
         return databases.DatabaseModule.NODE_TYPE
 
+    @property
+    def module_use_template_javascript(self):
+        """
+        Returns whether Jinja2 template is used for generating the javascript
+        module.
+        """
+        return False
+
 
 blueprint = ForeignServerModule(__name__)
 
@@ -92,9 +106,10 @@ class ForeignServerView(PGChildNodeView):
     """
     class ForeignServerView(PGChildNodeView)
 
-        A view class for foreign server node derived from PGChildNodeView. This class is
-        responsible for all the stuff related to view like updating foreign server
-        node, showing properties, showing sql in sql pane.
+        A view class for foreign server node derived from PGChildNodeView.
+        This class is responsible for all the stuff related to view like
+        updating foreign server node, showing properties,
+        showing sql in sql pane.
 
     Methods:
     -------
@@ -111,17 +126,21 @@ class ForeignServerView(PGChildNodeView):
         manager,conn & template_path properties to self
 
     * list(gid, sid, did, fid)
-      - This function is used to list all the foreign server nodes within that collection.
+      - This function is used to list all the foreign server nodes within
+        that collection.
 
     * nodes(gid, sid, did, fid)
-      - This function will used to create all the child node within that collection.
+      - This function will used to create all the child node within
+        that collection.
         Here it will create all the foreign server node.
 
     * properties(gid, sid, did, fid, fsid)
-      - This function will show the properties of the selected foreign server node
+      - This function will show the properties of the selected
+        foreign server node
 
     * update(gid, sid, did, fid, fsid)
-      - This function will update the data for the selected foreign server node
+      - This function will update the data for the selected
+        foreign server node
 
     * create(gid, sid, did, fid)
       - This function will create the new foreign server node
@@ -130,19 +149,23 @@ class ForeignServerView(PGChildNodeView):
       - This function will delete the selected foreign server node
 
     * msql(gid, sid, did, fid, fsid)
-      - This function is used to return modified SQL for the selected foreign server node
+      - This function is used to return modified SQL for the selected
+        foreign server node
 
     * get_sql(data, fid, fsid)
       - This function will generate sql from model data
 
     * sql(gid, sid, did, fid, fsid):
-      - This function will generate sql to show it in sql pane for the selected foreign server node.
+      - This function will generate sql to show it in sql pane for the
+        selected foreign server node.
 
     * dependents(gid, sid, did, fid, fsid):
-      - This function get the dependents and return ajax response for the foreign server node.
+      - This function get the dependents and return ajax response for the
+        foreign server node.
 
     * dependencies(self, gid, sid, did, fid, fsid):
-      - This function get the dependencies and return ajax response for the foreign server node.
+      - This function get the dependencies and return ajax response for the
+        foreign server node.
     """
 
     node_type = blueprint.node_type
@@ -160,11 +183,9 @@ class ForeignServerView(PGChildNodeView):
     operations = dict({
         'obj': [
             {'get': 'properties', 'delete': 'delete', 'put': 'update'},
-            {'get': 'list', 'post': 'create'}
+            {'get': 'list', 'post': 'create', 'delete': 'delete'}
         ],
-        'delete': [{
-            'delete': 'delete'
-        }],
+        'delete': [{'delete': 'delete'}, {'delete': 'delete'}],
         'nodes': [{'get': 'node'}, {'get': 'nodes'}],
         'children': [{'get': 'children'}],
         'sql': [{'get': 'sql'}],
@@ -216,7 +237,8 @@ class ForeignServerView(PGChildNodeView):
     @check_precondition
     def list(self, gid, sid, did, fid):
         """
-        This function is used to list all the foreign server nodes within that collection.
+        This function is used to list all the foreign server nodes
+        within that collection.
 
         Args:
             gid: Server Group ID
@@ -240,7 +262,8 @@ class ForeignServerView(PGChildNodeView):
     @check_precondition
     def nodes(self, gid, sid, did, fid):
         """
-        This function will used to create all the child node within that collection.
+        This function will used to create all the child node
+        within that collection.
         Here it will create all the foreign server node.
 
         Args:
@@ -310,7 +333,8 @@ class ForeignServerView(PGChildNodeView):
     @check_precondition
     def properties(self, gid, sid, did, fid, fsid):
         """
-        This function will show the properties of the selected foreign server node.
+        This function will show the properties of the selected
+        foreign server node.
 
         Args:
             gid: Server Group ID
@@ -337,7 +361,9 @@ class ForeignServerView(PGChildNodeView):
                 res['rows'][0]['fsrvoptions'], 'fsrvoption', 'fsrvvalue'
             )
 
-        sql = render_template("/".join([self.template_path, 'acl.sql']), fsid=fsid)
+        sql = render_template("/".join([self.template_path, 'acl.sql']),
+                              fsid=fsid
+                              )
         status, fs_rv_acl_res = self.conn.execute_dict(sql)
 
         if not status:
@@ -387,7 +413,8 @@ class ForeignServerView(PGChildNodeView):
             if 'fsrvacl' in data:
                 data['fsrvacl'] = parse_priv_to_db(data['fsrvacl'], ['U'])
 
-            sql = render_template("/".join([self.template_path, 'properties.sql']),
+            sql = render_template("/".join([self.template_path,
+                                            'properties.sql']),
                                   fdwid=fid, conn=self.conn)
             status, res1 = self.conn.execute_dict(sql)
             if not status:
@@ -409,7 +436,8 @@ class ForeignServerView(PGChildNodeView):
             if not status:
                 return internal_server_error(errormsg=res)
 
-            sql = render_template("/".join([self.template_path, 'properties.sql']),
+            sql = render_template("/".join([self.template_path,
+                                            'properties.sql']),
                                   data=data, fdwdata=fdw_data,
                                   conn=self.conn)
             status, r_set = self.conn.execute_dict(sql)
@@ -430,7 +458,8 @@ class ForeignServerView(PGChildNodeView):
     @check_precondition
     def update(self, gid, sid, did, fid, fsid):
         """
-        This function will update the data for the selected foreign server node.
+        This function will update the data for the selected
+        foreign server node.
 
         Args:
             gid: Server Group ID
@@ -446,6 +475,9 @@ class ForeignServerView(PGChildNodeView):
 
         try:
             sql, name = self.get_sql(gid, sid, data, did, fid, fsid)
+            # Most probably this is due to error
+            if not isinstance(sql, (str, unicode)):
+                return sql
             sql = sql.strip('\n').strip(' ')
             status, res = self.conn.execute_scalar(sql)
             if not status:
@@ -464,7 +496,7 @@ class ForeignServerView(PGChildNodeView):
             return internal_server_error(errormsg=str(e))
 
     @check_precondition
-    def delete(self, gid, sid, did, fid, fsid):
+    def delete(self, gid, sid, did, fid, fsid=None):
         """
         This function will delete the selected foreign server node.
 
@@ -476,6 +508,13 @@ class ForeignServerView(PGChildNodeView):
             fsid: foreign server ID
         """
 
+        if fsid is None:
+            data = request.form if request.form else json.loads(
+                request.data, encoding='utf-8'
+            )
+        else:
+            data = {'ids': [fsid]}
+
         if self.cmd == 'delete':
             # This is a cascade operation
             cascade = True
@@ -483,43 +522,40 @@ class ForeignServerView(PGChildNodeView):
             cascade = False
 
         try:
-            # Get name of foreign data wrapper from fid
-            sql = render_template("/".join([self.template_path, 'delete.sql']),
-                                  fsid=fsid, conn=self.conn)
-            status, name = self.conn.execute_scalar(sql)
-            if not status:
-                return internal_server_error(errormsg=name)
+            for fsid in data['ids']:
+                # Get name of foreign data wrapper from fid
+                sql = render_template("/".join([self.template_path,
+                                                'delete.sql']),
+                                      fsid=fsid, conn=self.conn)
+                status, name = self.conn.execute_scalar(sql)
+                if not status:
+                    return internal_server_error(errormsg=name)
 
-            if name is None:
-                return make_json_response(
-                    status=410,
-                    success=0,
-                    errormsg=gettext(
-                        'Error: Object not found.'
-                    ),
-                    info=gettext(
-                        'The specified foreign server could not be found.\n'
+                if name is None:
+                    return make_json_response(
+                        status=410,
+                        success=0,
+                        errormsg=gettext(
+                            'Error: Object not found.'
+                        ),
+                        info=gettext(
+                            'The specified foreign server '
+                            'could not be found.\n'
+                        )
                     )
-                )
 
-            # drop foreign server
-            sql = render_template("/".join([self.template_path, 'delete.sql']),
-                                  name=name, cascade=cascade,
-                                  conn=self.conn)
-            status, res = self.conn.execute_scalar(sql)
-            if not status:
-                return internal_server_error(errormsg=res)
+                # drop foreign server
+                sql = render_template("/".join([self.template_path,
+                                                'delete.sql']),
+                                      name=name, cascade=cascade,
+                                      conn=self.conn)
+                status, res = self.conn.execute_scalar(sql)
+                if not status:
+                    return internal_server_error(errormsg=res)
 
             return make_json_response(
                 success=1,
-                info=gettext("Foreign Server dropped"),
-                data={
-                    'id': fsid,
-                    'fid': fid,
-                    'did': did,
-                    'sid': sid,
-                    'gid': gid,
-                }
+                info=gettext("Foreign Server dropped")
             )
 
         except Exception as e:
@@ -528,7 +564,8 @@ class ForeignServerView(PGChildNodeView):
     @check_precondition
     def msql(self, gid, sid, did, fid, fsid=None):
         """
-        This function is used to return modified SQL for the selected foreign server node.
+        This function is used to return modified SQL for the selected foreign
+        server node.
 
         Args:
             gid: Server Group ID
@@ -546,13 +583,15 @@ class ForeignServerView(PGChildNodeView):
                 data[k] = v
         try:
             sql, name = self.get_sql(gid, sid, data, did, fid, fsid)
+            # Most probably this is due to error
+            if not isinstance(sql, (str, unicode)):
+                return sql
             if sql == '':
                     sql = "--modified SQL"
 
             return make_json_response(
                 data=sql.strip('\n'),
-                status=200
-                )
+                status=200)
 
         except Exception as e:
             return internal_server_error(errormsg=str(e))
@@ -575,11 +614,16 @@ class ForeignServerView(PGChildNodeView):
         ]
 
         if fsid is not None:
-            sql = render_template("/".join([self.template_path, 'properties.sql']),
+            sql = render_template("/".join([self.template_path,
+                                            'properties.sql']),
                                   fsid=fsid, conn=self.conn)
             status, res = self.conn.execute_dict(sql)
             if not status:
                 return internal_server_error(errormsg=res)
+            if len(res['rows']) == 0:
+                return gone(
+                    gettext("Could not find the foreign server information.")
+                )
 
             if res['rows'][0]['fsrvoptions'] is not None:
                 res['rows'][0]['fsrvoptions'] = tokenize_options(
@@ -589,11 +633,20 @@ class ForeignServerView(PGChildNodeView):
             for key in ['fsrvacl']:
                 if key in data and data[key] is not None:
                     if 'added' in data[key]:
-                        data[key]['added'] = parse_priv_to_db(data[key]['added'], ['U'])
+                        data[key]['added'] = parse_priv_to_db(
+                            data[key]['added'],
+                            ['U']
+                        )
                     if 'changed' in data[key]:
-                        data[key]['changed'] = parse_priv_to_db(data[key]['changed'], ['U'])
+                        data[key]['changed'] = parse_priv_to_db(
+                            data[key]['changed'],
+                            ['U']
+                        )
                     if 'deleted' in data[key]:
-                        data[key]['deleted'] = parse_priv_to_db(data[key]['deleted'], ['U'])
+                        data[key]['deleted'] = parse_priv_to_db(
+                            data[key]['deleted'],
+                            ['U']
+                        )
 
             old_data = res['rows'][0]
             for arg in required_args:
@@ -602,22 +655,31 @@ class ForeignServerView(PGChildNodeView):
 
             is_valid_added_options = is_valid_changed_options = False
             if 'fsrvoptions' in data and 'added' in data['fsrvoptions']:
-                is_valid_added_options, data['fsrvoptions']['added'] = validate_options(
-                    data['fsrvoptions']['added'], 'fsrvoption', 'fsrvvalue'
-                )
-            if 'fsrvoptions' in data and 'changed' in data['fsrvoptions']:
-                is_valid_changed_options, data['fsrvoptions']['changed'] = validate_options(
-                    data['fsrvoptions']['changed'], 'fsrvoption', 'fsrvvalue'
-                )
+                is_valid_added_options, data['fsrvoptions']['added'] =\
+                    validate_options(
+                        data['fsrvoptions']['added'],
+                        'fsrvoption',
+                        'fsrvvalue')
 
-            sql = render_template("/".join([self.template_path, 'update.sql']),
-                                  data=data, o_data=old_data,
-                                  is_valid_added_options=is_valid_added_options,
-                                  is_valid_changed_options=is_valid_changed_options,
-                                  conn=self.conn)
+            if 'fsrvoptions' in data and 'changed' in data['fsrvoptions']:
+                is_valid_changed_options, data['fsrvoptions']['changed'] =\
+                    validate_options(
+                        data['fsrvoptions']['changed'],
+                        'fsrvoption',
+                        'fsrvvalue')
+
+            sql = render_template(
+                "/".join([self.template_path, 'update.sql']),
+                data=data,
+                o_data=old_data,
+                is_valid_added_options=is_valid_added_options,
+                is_valid_changed_options=is_valid_changed_options,
+                conn=self.conn
+            )
             return sql, data['name'] if 'name' in data else old_data['name']
         else:
-            sql = render_template("/".join([self.template_path, 'properties.sql']),
+            sql = render_template("/".join([self.template_path,
+                                            'properties.sql']),
                                   fdwid=fid, conn=self.conn)
             status, res = self.conn.execute_dict(sql)
             if not status:
@@ -629,7 +691,6 @@ class ForeignServerView(PGChildNodeView):
                 if key in data and data[key] is not None:
                     data[key] = parse_priv_to_db(data[key], ['U'])
 
-
             is_valid_options = False
             if 'fsrvoptions' in data:
                 is_valid_options, data['fsrvoptions'] = validate_options(
@@ -643,11 +704,11 @@ class ForeignServerView(PGChildNodeView):
             sql += "\n"
         return sql, data['name']
 
-
     @check_precondition
     def sql(self, gid, sid, did, fid, fsid):
         """
-        This function will generate sql to show it in sql pane for the selected foreign server node.
+        This function will generate sql to show it in sql pane for the
+        selected foreign server node.
 
         Args:
             gid: Server Group ID
@@ -662,6 +723,10 @@ class ForeignServerView(PGChildNodeView):
         status, res = self.conn.execute_dict(sql)
         if not status:
             return internal_server_error(errormsg=res)
+        if len(res['rows']) == 0:
+            return gone(
+                gettext("Could not find the foreign server information.")
+            )
 
         is_valid_options = False
         if res['rows'][0]['fsrvoptions'] is not None:
@@ -672,7 +737,8 @@ class ForeignServerView(PGChildNodeView):
             if len(res['rows'][0]['fsrvoptions']) > 0:
                 is_valid_options = True
 
-        sql = render_template("/".join([self.template_path, 'acl.sql']), fsid=fsid)
+        sql = render_template("/".join([self.template_path, 'acl.sql']),
+                              fsid=fsid)
         status, fs_rv_acl_res = self.conn.execute_dict(sql)
         if not status:
             return internal_server_error(errormsg=fs_rv_acl_res)
@@ -686,7 +752,10 @@ class ForeignServerView(PGChildNodeView):
 
         # To format privileges
         if 'fsrvacl' in res['rows'][0]:
-            res['rows'][0]['fsrvacl'] = parse_priv_to_db(res['rows'][0]['fsrvacl'], ['U'])
+            res['rows'][0]['fsrvacl'] = parse_priv_to_db(
+                res['rows'][0]['fsrvacl'],
+                ['U']
+            )
 
         sql = render_template("/".join([self.template_path, 'properties.sql']),
                               fdwid=fid, conn=self.conn)
@@ -729,7 +798,8 @@ class ForeignServerView(PGChildNodeView):
         dependents_result = self.get_dependents(self.conn, fsid)
 
         # Fetching dependents of foreign servers
-        query = render_template("/".join([self.template_path, 'dependents.sql']),
+        query = render_template("/".join([self.template_path,
+                                          'dependents.sql']),
                                 fsid=fsid)
         status, result = self.conn.execute_dict(query)
         if not status:
